@@ -42,6 +42,8 @@ DEVICE_DIR_MAP = {
     'n300': 'n3xx',
     'n310': 'n3xx',
     'n320': 'n3xx',
+    'x400': 'x400',
+    'x410': 'x400',
 }
 
 # Picks the default make target per device
@@ -508,8 +510,8 @@ def convert_to_image_config(grc, grc_config_path):
         "schema": "rfnoc_imagebuilder",
         "copyright": "Ettus Research, A National Instruments Brand",
         "license": "SPDX-License-Identifier: LGPL-3.0-or-later",
-        "version": 1.0,
-        "rfnoc_version": 1.0}
+        "version": "1.0",
+        "rfnoc_version": "1.0"}
     # for param in [item for item in grc["blocks"] if item["id"] == "parameter"]:
     #     result[param["name"]] = {
     #         "str": lambda value: str,
@@ -522,8 +524,13 @@ def convert_to_image_config(grc, grc_config_path):
     result["stream_endpoints"] = {}
     for sep in seps.values():
         result["stream_endpoints"][sep["name"]] = {"ctrl": bool(sep["parameters"]["ctrl"]),
-                                                   "data": bool(sep["parameters"]["data"]),
-                                                   "buff_size": int(sep["parameters"]["buff_size"])}
+                                                   "data": bool(sep["parameters"]["data"])}
+        if "buff_size" in sep["parameters"]:
+            result["stream_endpoints"][sep["name"]]["buff_size"] = \
+                int(sep["parameters"]["buff_size"])
+        if "buff_size_bytes" in sep["parameters"]:
+            result["stream_endpoints"][sep["name"]]["buff_size_bytes"] = \
+                int(sep["parameters"]["buff_size_bytes"])
 
     result["noc_blocks"] = {}
     for block in blocks.values():
@@ -675,6 +682,8 @@ def write_verilog(config, destination, source, source_hash):
     template engine.
     :param config: ImageBuilderConfig derived from script parameter
     :param destination: Filepath to write to
+    :param source: Filepath to the image YAML/GRC to generate from
+    :param source_hash: Source file hash value
     :return: None
     """
     template_dir = os.path.join(os.path.dirname(__file__), "templates")
@@ -700,6 +709,38 @@ def write_verilog(config, destination, source, source_hash):
         image_core_file.write(block)
 
 
+def write_verilog_header(config, destination, source, source_hash):
+    """
+    Generates rfnoc_image_core.vh file for the device.
+    :param config: ImageBuilderConfig derived from script parameter
+    :param destination: Filepath to write to
+    :param source: Filepath to the image YAML/GRC to generate from
+    :param source_hash: Source file hash value
+    :return: None
+    """
+    template_dir = os.path.join(os.path.dirname(__file__), "templates")
+    lookup = mako.lookup.TemplateLookup(directories=[template_dir])
+    tpl_filename = os.path.join(template_dir, "rfnoc_image_core.vh.mako")
+    tpl = mako.template.Template(
+        filename=tpl_filename,
+        lookup=lookup,
+        strict_undefined=True)
+
+    try:
+        block = tpl.render(**{
+            "config": config,
+            "source": source,
+            "source_hash": source_hash,
+            })
+    except:
+        print(exceptions.text_error_template().render())
+        sys.exit(1)
+
+    logging.info("Writing image core header to %s", destination)
+    with open(destination, "w") as image_core_file:
+        image_core_file.write(block)
+
+
 def write_build_env():
     """
     # TODO update Makefile entries according to used blocks
@@ -795,18 +836,31 @@ def get_core_config_path(config_path):
     """
     return os.path.join(config_path, RFNOC_CORE_DIR)
 
-def generate_image_core_path(output_path, device, source):
+def generate_image_core_path(output_path, name, source):
     """
     Creates the path where the image core file gets to be stored.
 
     output_path: If not None, this is returned
-    device: Device type string, used to generate default file name
+    name: Device name string, used to generate default file name
     source: Otherwise, this path is returned, combined with a default file name
     """
     if output_path is not None:
-        return output_path
+        return os.path.splitext(output_path)[0] + '.v'
     source = os.path.split(os.path.abspath(os.path.normpath(source)))[0]
-    return os.path.join(source, "{}_rfnoc_image_core.v".format(device))
+    return os.path.join(source, "{}_rfnoc_image_core.v".format(name))
+
+def generate_image_core_header_path(output_path, name, source):
+    """
+    Creates the path where the image core header file will be stored.
+
+    output_path: If not None, this is returned
+    name: Device name string, used to generate default file name
+    source: Otherwise, this path is returned, combined with a default file name
+    """
+    if output_path is not None:
+        return os.path.splitext(output_path)[0] + '.vh'
+    source = os.path.split(os.path.abspath(os.path.normpath(source)))[0]
+    return os.path.join(source, "{}_rfnoc_image_core.vh".format(name))
 
 def generate_edge_file_path(output_path, device, source):
     """
@@ -828,7 +882,7 @@ def build_image(config, fpga_path, config_path, device, **args):
     requested.
 
     :param config: A dictionary containing the image configuration options.
-                   This must obey the rfnoc_imagebuilder_args schema.
+                   This must obey the rfnoc_image_builder_args schema.
     :param fpga_path: A path that holds the FPGA IP sources.
     :param device: The device to build for.
     :param **args: Additional options including
@@ -842,12 +896,16 @@ def build_image(config, fpga_path, config_path, device, **args):
     logging.info("Selected device %s", device)
     image_core_path = \
         generate_image_core_path(
-            args.get('output_path'), device, args.get('source'))
+            args.get('output_path'), args.get('image_core_name'), args.get('source'))
+    image_core_header_path = \
+        generate_image_core_header_path(
+            args.get('output_path'), args.get('image_core_name'), args.get('source'))
     edge_file = \
         generate_edge_file_path(
-            args.get('router_hex_path'), device, args.get('source'))
+            args.get('router_hex_path'), args.get('image_core_name'), args.get('source'))
 
     logging.debug("Image core output file: %s", image_core_path)
+    logging.debug("Image core header output file: %s", image_core_header_path)
     logging.debug("Edge output file: %s", edge_file)
 
     core_config_path = get_core_config_path(config_path)
@@ -869,6 +927,11 @@ def build_image(config, fpga_path, config_path, device, **args):
         image_core_path,
         source=args.get('source'),
         source_hash=args.get('source_hash'))
+    write_verilog_header(
+        builder_conf,
+        image_core_header_path,
+        source=args.get('source'),
+        source_hash=args.get('source_hash'))
     write_build_env()
 
     if "generate_only" in args and args["generate_only"]:
